@@ -6,7 +6,8 @@
 
 #include "DasherInterfaceBase.h"
 
-#include "CustomColours.h"
+//#include "ActionButton.h"
+#include "AlphabetManagerFactory.h"
 #include "DasherViewSquare.h"
 #include "ControlManager.h"
 #include "DasherScreen.h"
@@ -15,7 +16,9 @@
 #include "DasherModel.h"
 #include "EventHandler.h"
 #include "Event.h"
+#include "NodeCreationManager.h"
 #include "UserLog.h"
+#include "BasicLog.h"
 #include "WrapperFactory.h"
 
 // Input filters
@@ -24,7 +27,6 @@
 #include "DasherButtons.h"
 #include "DynamicFilter.h"
 #include "EyetrackerFilter.h"
-//#include "OneButtonFilter.h"
 #include "OneDimensionalFilter.h"
 #include "StylusFilter.h"
 #include "TwoButtonDynamicFilter.h"
@@ -63,12 +65,15 @@ static char THIS_FILE[] = __FILE__;
 #endif
 
 CDasherInterfaceBase::CDasherInterfaceBase()
-                  :m_Alphabet(0), m_pColours(0), m_pDasherModel(0), m_DasherScreen(0),
+                  :m_Alphabet(0), m_pDasherModel(0), m_DasherScreen(0),
 		   m_pDasherView(0), m_pInput(0), m_AlphIO(0), m_ColourIO(0), m_pUserLog(NULL),
 		   m_pInputFilter(NULL) {
+
+  m_pNCManager = 0;
   
   m_bGlobalLock = false;
   m_bShutdownLock = false;
+  m_bRecreateLock = false;
 
   m_bRedrawScheduled = false;
 
@@ -107,18 +112,27 @@ void CDasherInterfaceBase::Realize() {
   // until now so we have the real value of the parameter and not
   // just the default.
 
+  // TODO: Sort out log type selection
+
   int iUserLogLevel = GetLongParameter(LP_USER_LOG_LEVEL_MASK);
-  if (iUserLogLevel > 0) 
+
+  if(iUserLogLevel == 10)
+    m_pUserLog = new CBasicLog(m_pEventHandler, m_pSettingsStore);
+  else if (iUserLogLevel > 0) 
     m_pUserLog = new CUserLog(m_pEventHandler, m_pSettingsStore, iUserLogLevel, m_Alphabet);  
 
   CreateFactories();
   CreateInputFilter();
+  SetupActionButtons();
+
+  // FIXME - need to rationalise this sort of thing.
+  InvalidateContext(true);
+  ScheduleRedraw();
     
   // All the setup is done by now, so let the user log object know
   // that future parameter changes should be logged.
   if (m_pUserLog != NULL) 
     m_pUserLog->InitIsDone();
-  
 }
 
 CDasherInterfaceBase::~CDasherInterfaceBase() {
@@ -128,8 +142,8 @@ CDasherInterfaceBase::~CDasherInterfaceBase() {
   delete m_pDasherView;
   delete m_ColourIO;
   delete m_AlphIO;
-  delete m_pColours;
   delete m_pInputFilter;
+  delete m_pNCManager;
   // Do NOT delete Edit box or Screen. This class did not create them.
 
   // When we destruct on shutdown, we'll output any detailed log file
@@ -184,13 +198,17 @@ void CDasherInterfaceBase::InterfaceEventHandler(Dasher::CEvent *pEvent) {
 
     switch (pEvt->m_iParameter) {
 
-    case BP_COLOUR_MODE:       // Forces us to redraw the display
-      // TODO: Is this variable ever used any more?
-      Start();
-      Redraw(true);
-      break;
     case BP_OUTLINE_MODE:
-      Redraw(true);
+      ScheduleRedraw();
+      break;
+    case BP_DRAW_MOUSE:
+      ScheduleRedraw();
+      break;
+    case BP_CONTROL_MODE:
+      ScheduleRedraw();
+      break;
+    case BP_DRAW_MOUSE_LINE:
+      ScheduleRedraw();
       break;
     case LP_ORIENTATION:
       if(GetLongParameter(LP_ORIENTATION) == Dasher::Opts::AlphabetDefault)
@@ -198,16 +216,15 @@ void CDasherInterfaceBase::InterfaceEventHandler(Dasher::CEvent *pEvent) {
 	SetLongParameter(LP_REAL_ORIENTATION, m_Alphabet->GetOrientation());
       else
 	SetLongParameter(LP_REAL_ORIENTATION, GetLongParameter(LP_ORIENTATION));
-      Redraw(true);
+      ScheduleRedraw();
       break;
     case SP_ALPHABET_ID:
       ChangeAlphabet();
-      Start();
-      Redraw(true);
+      ScheduleRedraw();
       break;
     case SP_COLOUR_ID:
       ChangeColours();
-      Redraw(true);
+      ScheduleRedraw();
       break;
     case SP_DEFAULT_COLOUR_ID: // Delibarate fallthrough
     case BP_PALETTE_CHANGE: 
@@ -216,21 +233,19 @@ void CDasherInterfaceBase::InterfaceEventHandler(Dasher::CEvent *pEvent) {
       break;
     case LP_LANGUAGE_MODEL_ID:
       CreateDasherModel();
-      Start();
-      Redraw(true);
       break;
     case LP_LINE_WIDTH:
-      Redraw(false); // TODO - make this accessible everywhere
+      ScheduleRedraw();
       break;
     case LP_DASHER_FONTSIZE:
-      // TODO - make screen a CDasherComponent child?
-      Redraw(true);
+      ScheduleRedraw();
       break;
     case SP_INPUT_DEVICE:
       CreateInput();
       break;
     case SP_INPUT_FILTER:
       CreateInputFilter();
+      ScheduleRedraw();
       break;
     default:
       break;
@@ -286,7 +301,7 @@ void CDasherInterfaceBase::CreateDasherModel()
 {
   if(!m_AlphIO)
     return;
-  
+
   // TODO: Move training into model?
   // TODO: Do we really need to check for a valid language model?
   int lmID = GetLongParameter(LP_LANGUAGE_MODEL_ID);
@@ -299,20 +314,30 @@ void CDasherInterfaceBase::CreateDasherModel()
     m_pEventHandler->InsertEvent(pEvent);
     delete pEvent;
 
+    m_bRecreateLock = true;
+
     // Delete the old model and create a new one
     if(m_pDasherModel) {
       delete m_pDasherModel;
       m_pDasherModel = 0;
     }
 
-    if(m_deGameModeStrings.size() == 0)
-      m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, this, m_AlphIO);
+    if(m_pNCManager) {
+      delete m_pNCManager;
+      m_pNCManager = 0;
+    }
+      
+    if(m_deGameModeStrings.size() == 0) {
+      m_pNCManager = new CNodeCreationManager(m_pEventHandler, m_pSettingsStore, false, "", m_AlphIO);
+      m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, m_pNCManager, this);
+    }
     else {
-      m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, this, m_AlphIO, true, m_deGameModeStrings[0]);
+      m_pNCManager = new CNodeCreationManager(m_pEventHandler, m_pSettingsStore, true, m_deGameModeStrings[0], m_AlphIO);
+      m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, m_pNCManager, this, true, m_deGameModeStrings[0]);
       //      m_deGameModeStrings.pop_front();
     }
 
-    m_Alphabet = m_pDasherModel->GetAlphabetNew();
+    m_Alphabet = m_pNCManager->GetAlphabet();
     
     string T = m_Alphabet->GetTrainingFile();
 
@@ -334,6 +359,11 @@ void CDasherInterfaceBase::CreateDasherModel()
     m_pEventHandler->InsertEvent(pEvent);
     delete pEvent;
   }
+
+  Start();
+  Redraw(true);
+
+  m_bRecreateLock = false;
 }
 
 void CDasherInterfaceBase::Start() {
@@ -345,9 +375,15 @@ void CDasherInterfaceBase::Start() {
     m_pDasherModel->Start();
   }
   if(m_pDasherView != 0) {
-    m_pDasherView->ResetSum();
-    m_pDasherView->ResetSumCounter();
-    m_pDasherView->ResetYAutoOffset();
+//     m_pDasherView->ResetSum();
+//     m_pDasherView->ResetSumCounter();
+//     m_pDasherView->ResetYAutoOffset();
+  }
+
+  int iMinWidth;
+  
+  if(m_pInputFilter && m_pInputFilter->GetMinWidth(iMinWidth)) {
+    m_pDasherModel->LimitRoot(iMinWidth);
   }
 }
 
@@ -367,9 +403,11 @@ void CDasherInterfaceBase::PauseAt(int MouseX, int MouseY) {
 void CDasherInterfaceBase::Halt() {
   SetBoolParameter(BP_DASHER_PAUSED, true);
 
+  // Obsolete?
   if(GetBoolParameter(BP_MOUSEPOS_MODE)) {
     SetLongParameter(LP_MOUSE_POS_BOX, 1);
   }
+  //
 
   // This will cause us to reinitialise the frame rate counter - ie we start off slowly
   if(m_pDasherModel != 0)
@@ -384,8 +422,8 @@ void CDasherInterfaceBase::Unpause(unsigned long Time) {
     //m_pDasherModel->Set_paused(m_Paused);
   }
   if(m_pDasherView != 0) {
-    m_pDasherView->ResetSum();
-    m_pDasherView->ResetSumCounter();
+//     m_pDasherView->ResetSum();
+//     m_pDasherView->ResetSumCounter();
   }
 
   Dasher::CStartEvent oEvent;
@@ -393,13 +431,10 @@ void CDasherInterfaceBase::Unpause(unsigned long Time) {
 
   ResetNats();
   if (m_pUserLog != NULL)
-	  m_pUserLog->StartWriting();
+    m_pUserLog->StartWriting();
 }
 
 void CDasherInterfaceBase::CreateInput() {
-
-  // FIXME - this shouldn't be the model used here - we should just change a parameter and work from the appropriate listener
-
   if(m_pInput) {
     m_pInput->Deactivate();
     m_pInput->Unref();
@@ -416,9 +451,9 @@ void CDasherInterfaceBase::CreateInput() {
     m_pDasherView->SetInput(m_pInput);
 }
 
-void CDasherInterfaceBase::NewFrame(unsigned long iTime) {
+void CDasherInterfaceBase::NewFrame(unsigned long iTime, bool bForceRedraw) {
   // Fail if Dasher is locked
-  if(m_bGlobalLock || m_bShutdownLock)
+  if(m_bGlobalLock || m_bShutdownLock || m_bRecreateLock)
     return;
 
   bool bChanged(false);
@@ -431,7 +466,7 @@ void CDasherInterfaceBase::NewFrame(unsigned long iTime) {
 	int iNumDeleted = 0;
 	
 	if(m_pInputFilter) {
-	  bChanged = m_pInputFilter->Timer(iTime, m_pDasherView, m_pDasherModel); // FIXME - need logging stuff here
+	  bChanged = m_pInputFilter->Timer(iTime, m_pDasherView, m_pDasherModel, &vAdded, &iNumDeleted);
 	}
 	
 	if (iNumDeleted > 0)
@@ -442,7 +477,7 @@ void CDasherInterfaceBase::NewFrame(unsigned long iTime) {
       }
       else {
 	if(m_pInputFilter) {
-	  bChanged = m_pInputFilter->Timer(iTime, m_pDasherView, m_pDasherModel);
+	  bChanged = m_pInputFilter->Timer(iTime, m_pDasherView, m_pDasherModel, 0, 0);
 	}
       }
       
@@ -450,7 +485,7 @@ void CDasherInterfaceBase::NewFrame(unsigned long iTime) {
     }
   }
 
-  Redraw(bChanged || m_bRedrawScheduled);
+  Redraw(bChanged || m_bRedrawScheduled || bForceRedraw);
   m_bRedrawScheduled = false;
 
   // This just passes the time through to the framerate tracker, so we
@@ -459,6 +494,13 @@ void CDasherInterfaceBase::NewFrame(unsigned long iTime) {
     m_pDasherModel->NewFrame(iTime);
 }
 
+void CDasherInterfaceBase::CheckRedraw() {
+  if(m_bRedrawScheduled)
+    Redraw(true);
+
+  m_bRedrawScheduled = false;
+}
+
 /// Full redraw - renders model, decorations and blits onto display
 /// buffer. Not recommended if nothing has changed with the model,
 /// otherwise we're wasting effort.
@@ -467,6 +509,8 @@ void CDasherInterfaceBase::Redraw(bool bRedrawNodes) {
   if(!m_pDasherView || !m_pDasherModel)
     return;
   
+  // TODO: More generic notion of 'layers'?
+
   if(bRedrawNodes) {
     m_pDasherView->Screen()->SendMarker(0);
     m_pDasherModel->RenderToView(m_pDasherView, true);
@@ -475,12 +519,17 @@ void CDasherInterfaceBase::Redraw(bool bRedrawNodes) {
   m_pDasherView->Screen()->SendMarker(1);
   
   bool bDecorationsChanged(false);
-  
+
   if(m_pInputFilter) {
     bDecorationsChanged = m_pInputFilter->DecorateView(m_pDasherView);
   }
-  
-  if(bRedrawNodes || bDecorationsChanged)
+
+  bool bActionButtonsChanged(false);
+#ifdef EXPERIMENTAL_FEATURES
+  bActionButtonsChanged = DrawActionButtons();
+#endif
+
+  if(bRedrawNodes || bDecorationsChanged || bActionButtonsChanged)
     m_pDasherView->Display();
 }
 
@@ -522,39 +571,20 @@ void CDasherInterfaceBase::ChangeAlphabet() {
 
   SetBoolParameter( BP_TRAINING, false );
 
-  Start();
-
   //}
 }
 
-// std::string CDasherInterfaceBase::GetCurrentAlphabet() {
-//   return GetStringParameter(SP_ALPHABET_ID);
-// }
-
 void CDasherInterfaceBase::ChangeColours() {
-  if(!m_ColourIO)
+  if(!m_ColourIO || !m_DasherScreen)
     return;
-
-//   // delete old colours on editing function (ending function? - not really sure what the point of this is - I guess we might fail)
-//   std::auto_ptr < CCustomColours > ptrColours(m_pColours);
-
-  if(m_pColours) {
-    delete m_pColours;
-    m_pColours = 0;
-  }
-
  
-  CColourIO::ColourInfo oColourInfo(m_ColourIO->GetInfo(GetStringParameter(SP_COLOUR_ID)));
-  m_pColours = new CCustomColours(oColourInfo);
-
-  if(m_DasherScreen != 0) {
-    m_DasherScreen->SetColourScheme(m_pColours);
-  }
+  // TODO: Make fuction return a pointer directly
+  m_DasherScreen->SetColourScheme(&(m_ColourIO->GetInfo(GetStringParameter(SP_COLOUR_ID))));
 }
 
 void CDasherInterfaceBase::ChangeScreen(CDasherScreen *NewScreen) {
   m_DasherScreen = NewScreen;
-  m_DasherScreen->SetColourScheme(m_pColours);
+  ChangeColours();
 
   if(m_pDasherView != 0) {
     m_pDasherView->ChangeScreen(m_DasherScreen);
@@ -563,22 +593,20 @@ void CDasherInterfaceBase::ChangeScreen(CDasherScreen *NewScreen) {
       ChangeView();
   }
 
-  Redraw(true);
+  PositionActionButtons();
+  ScheduleRedraw();
 }
 
 void CDasherInterfaceBase::ChangeView() {
   // TODO: Actually respond to LP_VIEW_ID parameter (although there is only one view at the moment)
 
-  if(m_DasherScreen != 0 && m_pDasherModel != 0) 
-  {
-	  delete m_pDasherView;
-// 	  if(m_pAutoSpeedControl)
-// 	    delete m_pAutoSpeedControl;
- 	  m_pDasherView = new CDasherViewSquare(m_pEventHandler, m_pSettingsStore, m_DasherScreen);
-
-
-	  if (m_pInput)
-	    m_pDasherView->SetInput(m_pInput);
+  if(m_DasherScreen != 0 && m_pDasherModel != 0) {
+    delete m_pDasherView;
+    
+    m_pDasherView = new CDasherViewSquare(m_pEventHandler, m_pSettingsStore, m_DasherScreen);
+ 
+    if (m_pInput)
+      m_pDasherView->SetInput(m_pInput);
   }
 }
 
@@ -602,26 +630,6 @@ void CDasherInterfaceBase::GetColours(std::vector <std::string >*ColourList) {
   m_ColourIO->GetColours(ColourList);
 }
 
-int CDasherInterfaceBase::TrainFileWrapper(string Filename, int iTotalBytes, int iOffset) {
-   // Train the new language model
-    CLockEvent *pEvent;
-    
-    pEvent = new CLockEvent("Training Dasher", true, 0);
-    m_pEventHandler->InsertEvent(pEvent);
-    delete pEvent;
-
-  int iRetVal = TrainFile(Filename, iTotalBytes, iOffset);
-   // Train the new language model
-    
-    
-    pEvent = new CLockEvent("Training Dasher", false, 0);
-    m_pEventHandler->InsertEvent(pEvent);
-    delete pEvent;
-
-
-    return iRetVal;
-}
-
 /*
 	I've used C style I/O because I found that C++ style I/O bloated
 	the Win32 code enormously. The overhead of loading the buffer into
@@ -644,7 +652,7 @@ int CDasherInterfaceBase::TrainFile(string Filename, int iTotalBytes, int iOffse
 
   vector < symbol > Symbols;
 
-  CDasherModel::CTrainer * pTrainer = m_pDasherModel->GetTrainer();
+  CAlphabetManagerFactory::CTrainer * pTrainer = m_pDasherModel->GetTrainer();
   do {
     NumberRead = fread(InputBuffer, 1, BufferSize - 1, InputFile);
     InputBuffer[NumberRead] = '\0';
@@ -696,12 +704,12 @@ double CDasherInterfaceBase::GetCurFPS() {
   return 0;
 }
 
-int CDasherInterfaceBase::GetAutoOffset() {
-  if(m_pDasherView != 0) {
-    return m_pDasherView->GetAutoOffset();
-  }
-  return -1;
-}
+// int CDasherInterfaceBase::GetAutoOffset() {
+//   if(m_pDasherView != 0) {
+//     return m_pDasherView->GetAutoOffset();
+//   }
+//   return -1;
+// }
 
 double CDasherInterfaceBase::GetNats() const {
   if(m_pDasherModel)
@@ -738,7 +746,9 @@ void CDasherInterfaceBase::InvalidateContext(bool bForceStart) {
    if(bForceStart || (strNewContext.substr( std::max(static_cast<int>(strNewContext.size()) - iContextLength, 0)) != strCurrentContext.substr( std::max(static_cast<int>(strCurrentContext.size()) - iContextLength, 0)))) {
 
      if(m_pDasherModel != NULL) {
-       if(m_pDasherModel->m_bContextSensitive || bForceStart) {
+       // TODO: Reimplement this
+       //       if(m_pDasherModel->m_bContextSensitive || bForceStart) {
+       {
  	m_pDasherModel->SetContext(strNewContext);
  	PauseAt(0,0);
        }
@@ -748,18 +758,22 @@ void CDasherInterfaceBase::InvalidateContext(bool bForceStart) {
      WriteTrainFileFull();
    }
 
-  
+   if(bForceStart) {
+     int iMinWidth;
+
+     if(m_pInputFilter && m_pInputFilter->GetMinWidth(iMinWidth)) {
+       m_pDasherModel->LimitRoot(iMinWidth);
+     }
+   }
 
    if(m_pDasherView)
      while( m_pDasherModel->CheckForNewRoot(m_pDasherView) ) {
        // Do nothing
      }
 
-   //   Redraw(true);
    ScheduleRedraw();
 }
 
-
 void CDasherInterfaceBase::SetContext(std::string strNewContext) {
   m_pDasherModel->m_strContextBuffer = strNewContext;
 }
@@ -767,15 +781,15 @@ void CDasherInterfaceBase::SetContext(std::string strNewContext) {
 // Control mode stuff
 
 void CDasherInterfaceBase::RegisterNode( int iID, const std::string &strLabel, int iColour ) {
-  m_pDasherModel->RegisterNode(iID, strLabel, iColour);
+  m_pNCManager->RegisterNode(iID, strLabel, iColour);
 }
 
 void CDasherInterfaceBase::ConnectNode(int iChild, int iParent, int iAfter) {
-  m_pDasherModel->ConnectNode(iChild, iParent, iAfter);
+  m_pNCManager->ConnectNode(iChild, iParent, iAfter);
 }
 
 void CDasherInterfaceBase::DisconnectNode(int iChild, int iParent) {
-  m_pDasherModel->DisconnectNode(iChild, iParent);
+  m_pNCManager->DisconnectNode(iChild, iParent);
 }
 
 void CDasherInterfaceBase::SetBoolParameter(int iParameter, bool bValue) {
@@ -808,17 +822,23 @@ void CDasherInterfaceBase::ResetParameter(int iParameter) {
 }
 
 // We need to be able to get at the UserLog object from outside the interface
-CUserLog* CDasherInterfaceBase::GetUserLogPtr() {
+CUserLogBase* CDasherInterfaceBase::GetUserLogPtr() {
   return m_pUserLog;
 }
 
 void CDasherInterfaceBase::KeyDown(int iTime, int iId) {
+  if(m_bGlobalLock || m_bShutdownLock || m_bRecreateLock)
+    return;
+
   if(m_pInputFilter && !GetBoolParameter(BP_TRAINING)) {
-    m_pInputFilter->KeyDown(iTime, iId, m_pDasherModel);
+    m_pInputFilter->KeyDown(iTime, iId, m_pDasherModel, m_pUserLog);
   }
 }
 
 void CDasherInterfaceBase::KeyUp(int iTime, int iId) {
+  if(m_bGlobalLock || m_bShutdownLock || m_bRecreateLock)
+    return;
+
   if(m_pInputFilter && !GetBoolParameter(BP_TRAINING)) {
     m_pInputFilter->KeyUp(iTime, iId, m_pDasherModel);
   }
@@ -858,7 +878,6 @@ CDasherModule *CDasherInterfaceBase::GetModuleByName(const std::string &strName)
    return m_oModuleManager.GetModuleByName(strName);
 }
 
-
 void CDasherInterfaceBase::CreateFactories() {
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDefaultFilter(m_pEventHandler, m_pSettingsStore, this, m_pDasherModel,3, "Normal Control")));
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new COneDimensionalFilter(m_pEventHandler, m_pSettingsStore, this, m_pDasherModel)));
@@ -901,3 +920,115 @@ void CDasherInterfaceBase::StartShutdown() {
 bool CDasherInterfaceBase::GetModuleSettings(const std::string &strName, SModuleSettings **pSettings, int *iCount) {
   return GetModuleByName(strName)->GetSettings(pSettings, iCount);
 }
+
+void CDasherInterfaceBase::SetupActionButtons() {
+  m_vLeftButtons.push_back(new CActionButton(this, "Exit", true));
+  m_vLeftButtons.push_back(new CActionButton(this, "Preferences", false));
+  m_vLeftButtons.push_back(new CActionButton(this, "Help", false));
+  m_vLeftButtons.push_back(new CActionButton(this, "About", false));
+}
+
+void CDasherInterfaceBase::DestroyActionButtons() {
+  // TODO: implement and call this
+}
+
+void CDasherInterfaceBase::PositionActionButtons() {
+  if(!m_DasherScreen)
+    return;
+
+  int iCurrentOffset(16);
+
+  for(std::vector<CActionButton *>::iterator it(m_vLeftButtons.begin()); it != m_vLeftButtons.end(); ++it) {
+    (*it)->SetPosition(16, iCurrentOffset, 32, 32);
+    iCurrentOffset += 48;
+  }
+
+  iCurrentOffset = 16;
+
+  for(std::vector<CActionButton *>::iterator it(m_vRightButtons.begin()); it != m_vRightButtons.end(); ++it) {
+    (*it)->SetPosition(m_DasherScreen->GetWidth() - 144, iCurrentOffset, 128, 32);
+    iCurrentOffset += 48;
+  }
+}
+
+bool CDasherInterfaceBase::DrawActionButtons() {
+  if(!m_DasherScreen)
+    return false;
+
+  bool bVisible(GetBoolParameter(BP_DASHER_PAUSED));
+
+  bool bRV(bVisible != m_bOldVisible);
+  m_bOldVisible = bVisible;
+
+  for(std::vector<CActionButton *>::iterator it(m_vLeftButtons.begin()); it != m_vLeftButtons.end(); ++it)
+    (*it)->Draw(m_DasherScreen, bVisible);
+
+  for(std::vector<CActionButton *>::iterator it(m_vRightButtons.begin()); it != m_vRightButtons.end(); ++it)
+    (*it)->Draw(m_DasherScreen, bVisible);
+
+  return bRV;
+}
+
+
+void CDasherInterfaceBase::HandleClickUp(int iTime, int iX, int iY) {
+#ifdef EXPERIMENTAL_FEATURES
+  bool bVisible(GetBoolParameter(BP_DASHER_PAUSED));
+
+  for(std::vector<CActionButton *>::iterator it(m_vLeftButtons.begin()); it != m_vLeftButtons.end(); ++it) {
+    if((*it)->HandleClickUp(iTime, iX, iY, bVisible))
+      return;
+  }
+
+  for(std::vector<CActionButton *>::iterator it(m_vRightButtons.begin()); it != m_vRightButtons.end(); ++it) {
+    if((*it)->HandleClickUp(iTime, iX, iY, bVisible))
+      return;
+  }
+#endif
+
+  KeyUp(iTime, 100);
+}
+
+void CDasherInterfaceBase::HandleClickDown(int iTime, int iX, int iY) {
+#ifdef EXPERIMENTAL_FEATURES
+  bool bVisible(GetBoolParameter(BP_DASHER_PAUSED));
+
+  for(std::vector<CActionButton *>::iterator it(m_vLeftButtons.begin()); it != m_vLeftButtons.end(); ++it) {
+    if((*it)->HandleClickDown(iTime, iX, iY, bVisible))
+      return;
+  }
+
+  for(std::vector<CActionButton *>::iterator it(m_vRightButtons.begin()); it != m_vRightButtons.end(); ++it) {
+    if((*it)->HandleClickDown(iTime, iX, iY, bVisible))
+      return;
+  }
+#endif
+
+  KeyDown(iTime, 100);
+}
+
+
+void CDasherInterfaceBase::ExecuteCommand(const std::string &strName) {
+  // TODO: Pointless - just insert event directly
+
+  CCommandEvent *pEvent = new CCommandEvent(strName);
+  m_pEventHandler->InsertEvent(pEvent);
+  delete pEvent;
+}
+
+double CDasherInterfaceBase::GetFramerate() {
+  if(m_pDasherModel)
+    return(m_pDasherModel->Framerate());
+  else
+    return 0.0;
+}
+
+int CDasherInterfaceBase::GetRenderCount() {
+  if(m_pDasherView)
+    return(m_pDasherView->GetRenderCount());
+  else
+    return 0;
+}
+
+ void CDasherInterfaceBase::AddActionButton(const std::string &strName) {
+  m_vRightButtons.push_back(new CActionButton(this, strName, false));
+ }
