@@ -1,4 +1,5 @@
 #include "../Common/Common.h"
+#include "../../config.h"
 
 #include <iostream>
 #include "DasherControl.h"
@@ -18,15 +19,15 @@ using namespace std;
 extern "C" gint key_release_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
 extern "C" gboolean button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer data);
 extern "C" void realize_canvas(GtkWidget *widget, gpointer user_data);
-extern "C" void speed_changed(GtkHScale *hscale, gpointer user_data);
 extern "C" gint canvas_configure_event(GtkWidget *widget, GdkEventConfigure *event, gpointer data);
 extern "C" gint key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer data);
 extern "C" void canvas_destroy_event(GtkWidget *pWidget, gpointer pUserData);
-extern "C" void alphabet_combo_changed(GtkWidget *pWidget, gpointer pUserData);
 extern "C" gboolean canvas_focus_event(GtkWidget *widget, GdkEventFocus *event, gpointer data);
+extern "C" gint canvas_expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer data);
+
+static bool g_bHaveTimer = false;
 
 // CDasherControl class definitions
-
 CDasherControl::CDasherControl(GtkVBox *pVBox, GtkDasherControl *pDasherControl) {
   m_pDasherControl = pDasherControl;
   m_pVBox = GTK_WIDGET(pVBox);
@@ -34,11 +35,6 @@ CDasherControl::CDasherControl(GtkVBox *pVBox, GtkDasherControl *pDasherControl)
   Realize();
 
   m_iComboCount = 0;
-  PopulateAlphabetCombol();
-    
-  // Start the dasher model
-
-  Start();        // FIXME - should we hold off on this until later?
 
   // Create input device objects
   // (We create the SocketInput object now even if socket input is not enabled, because
@@ -48,9 +44,7 @@ CDasherControl::CDasherControl(GtkVBox *pVBox, GtkDasherControl *pDasherControl)
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherMouseInput(m_pEventHandler, m_pSettingsStore)));
   RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CSocketInput(m_pEventHandler, m_pSettingsStore)));
 
-  std::cout << "Settings store: " << m_pSettingsStore << std::endl;
-
-  m_pKeyboardHelper = new CKeyboardHelper;
+  m_pKeyboardHelper = new CKeyboardHelper(this);
   m_pKeyboardHelper->Grab(GetBoolParameter(BP_GLOBAL_KEYBOARD));
 
   CreateInput();
@@ -63,6 +57,13 @@ CDasherControl::CDasherControl(GtkVBox *pVBox, GtkDasherControl *pDasherControl)
 
   // Create a pango cache
 
+  m_pPangoCache = 0;
+
+  // TODO: Use system defaults?
+  if(GetStringParameter(SP_DASHER_FONT) == "")
+    SetStringParameter(SP_DASHER_FONT, "Sans 10");
+
+ 
   m_pPangoCache = new CPangoCache(GetStringParameter(SP_DASHER_FONT));
 
   // Don't create the screen until we've been realised.
@@ -79,38 +80,38 @@ void CDasherControl::SetupUI() {
   GtkWidget *pFrame = gtk_frame_new(NULL);
   gtk_frame_set_shadow_type(GTK_FRAME(pFrame), GTK_SHADOW_IN); 
   
-  m_pSpeedFrame = gtk_frame_new("Speed:");
+//   m_pSpeedFrame = gtk_frame_new("Speed:");
 
-  m_pSpeedHScale = gtk_hscale_new_with_range(0.1, 8.0, 0.1);
-  gtk_scale_set_digits( GTK_SCALE(m_pSpeedHScale), 1 );
+//   m_pSpeedHScale = gtk_hscale_new_with_range(0.1, 8.0, 0.1);
+//   gtk_scale_set_digits( GTK_SCALE(m_pSpeedHScale), 1 );
 
-  gtk_container_add(GTK_CONTAINER(m_pSpeedFrame), m_pSpeedHScale);
+//   gtk_container_add(GTK_CONTAINER(m_pSpeedFrame), m_pSpeedHScale);
   gtk_container_add(GTK_CONTAINER(pFrame), m_pCanvas);
 
 
-  m_pStatusBar = gtk_hbox_new(false, 2);
+//   m_pStatusBar = gtk_hbox_new(false, 2);
 
-  m_pSpin = gtk_spin_button_new_with_range(0.1, 8.0, 0.1);
-  m_pCombo = gtk_combo_box_new_text();
+//   m_pSpin = gtk_spin_button_new_with_range(0.1, 8.0, 0.1);
+//   m_pCombo = gtk_combo_box_new_text();
 
-  gtk_widget_set_size_request(m_pCombo, 256, -1);
+//   gtk_widget_set_size_request(m_pCombo, 256, -1);
 
-  m_pStatusLabel = gtk_label_new("Characters/min: --");
-  gtk_label_set_justify(GTK_LABEL(m_pStatusLabel), GTK_JUSTIFY_RIGHT);
+//   m_pStatusLabel = gtk_label_new("Characters/min: --");
+//   gtk_label_set_justify(GTK_LABEL(m_pStatusLabel), GTK_JUSTIFY_RIGHT);
 
-  gtk_box_pack_start(GTK_BOX(m_pStatusBar), gtk_label_new("Speed:"), 0, 0, 0);
-  gtk_box_pack_start(GTK_BOX(m_pStatusBar), m_pSpin, 0, 0, 0);
-  gtk_box_pack_start(GTK_BOX(m_pStatusBar), m_pCombo, 0, 0, 0);
-  //  gtk_box_pack_start(GTK_BOX(m_pStatusBar), m_pStatusLabel, TRUE, TRUE, 0);
+//   gtk_box_pack_start(GTK_BOX(m_pStatusBar), gtk_label_new("Speed:"), 0, 0, 0);
+//   //  gtk_box_pack_start(GTK_BOX(m_pStatusBar), m_pSpin, 0, 0, 0);
+//   //  gtk_box_pack_start(GTK_BOX(m_pStatusBar), m_pCombo, 0, 0, 0);
+//   //  gtk_box_pack_start(GTK_BOX(m_pStatusBar), m_pStatusLabel, TRUE, TRUE, 0);
 
-  gtk_box_pack_start(GTK_BOX(m_pVBox), pFrame, TRUE, TRUE, 0);
-  //  gtk_box_pack_start(GTK_BOX(m_pVBox), m_pSpeedFrame, FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(m_pVBox), m_pStatusBar, FALSE, FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(m_pVBox), pFrame, TRUE, TRUE, 0);
+//   //  gtk_box_pack_start(GTK_BOX(m_pVBox), m_pSpeedFrame, FALSE, FALSE, 0);
+//   gtk_box_pack_start(GTK_BOX(m_pVBox), m_pStatusBar, FALSE, FALSE, 0);
 
   gtk_widget_show_all(GTK_WIDGET(m_pVBox));
 
-  if(!GetBoolParameter(BP_SHOW_SLIDER))
-    gtk_widget_hide(m_pStatusBar);
+//   if(!GetBoolParameter(BP_SHOW_SLIDER))
+//     gtk_widget_hide(m_pStatusBar);
 
   // Connect callbacks - note that we need to implement the callbacks
   // as "C" style functions and pass this as user data so they can
@@ -118,23 +119,21 @@ void CDasherControl::SetupUI() {
 
   g_signal_connect(m_pCanvas, "button_press_event", G_CALLBACK(button_press_event), this);
   g_signal_connect(m_pCanvas, "button_release_event", G_CALLBACK(button_press_event), this);
-  g_signal_connect(m_pSpin, "value-changed", G_CALLBACK(speed_changed), this);
   g_signal_connect_after(m_pCanvas, "realize", G_CALLBACK(realize_canvas), this);
   g_signal_connect(m_pCanvas, "configure_event", G_CALLBACK(canvas_configure_event), this);
   g_signal_connect(m_pCanvas, "destroy", G_CALLBACK(canvas_destroy_event), this);
 
-  g_signal_connect(m_pCombo, "changed", G_CALLBACK(alphabet_combo_changed), this);
-
   // We'll use the same call back for keyboard events from the canvas
   // and slider - maybe this isn't the right thing to do long term
 
   g_signal_connect(m_pCanvas, "key-release-event", G_CALLBACK(key_release_event), this);
   g_signal_connect(m_pCanvas, "key_press_event", G_CALLBACK(key_press_event), this);
-  g_signal_connect(m_pSpeedHScale, "key_press_event", G_CALLBACK(key_press_event), this);
 
   g_signal_connect(m_pCanvas, "focus_in_event", G_CALLBACK(canvas_focus_event), this);
+  g_signal_connect(m_pCanvas, "expose_event", G_CALLBACK(canvas_expose_event), this);
 }
 
+
 void CDasherControl::SetupPaths() {
   char *home_dir;
   char *user_data_dir;
@@ -147,11 +146,7 @@ void CDasherControl::SetupPaths() {
   mkdir(user_data_dir, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
 
   // PROGDATA is provided by the makefile
-#ifdef WITH_MAEMO
-  system_data_dir = "/var/lib/install" PROGDATA "/";
-#else
   system_data_dir = PROGDATA "/";
-#endif
 
   SetStringParameter(SP_SYSTEM_LOC, system_data_dir);
   SetStringParameter(SP_USER_LOC, user_data_dir);
@@ -168,20 +163,26 @@ void CDasherControl::ScanAlphabetFiles(std::vector<std::string> &vFileList) {
   alphabetglob = g_pattern_spec_new("alphabet*xml");
 
   directory = g_dir_open(GetStringParameter(SP_SYSTEM_LOC).c_str(), 0, NULL);
-  while((filename = g_dir_read_name(directory))) {
-    if(alphabet_filter(filename, alphabetglob)) {
-      vFileList.push_back(filename);
+
+  if(directory) {
+    while((filename = g_dir_read_name(directory))) {
+      if(alphabet_filter(filename, alphabetglob)) {
+	vFileList.push_back(filename);
+      }
     }
+    g_dir_close(directory);
   }
-  g_dir_close(directory);
 
   directory = g_dir_open(GetStringParameter(SP_USER_LOC).c_str(), 0, NULL);
-  while((filename = g_dir_read_name(directory))) {
-    if(alphabet_filter(filename, alphabetglob)) {
-      vFileList.push_back(filename);
+
+  if(directory) {
+    while((filename = g_dir_read_name(directory))) {
+      if(alphabet_filter(filename, alphabetglob)) {
+	vFileList.push_back(filename);
+      }
     }
+    g_dir_close(directory);
   }
-  g_dir_close(directory);
   // FIXME - need to delete glob?
 }
 
@@ -193,26 +194,31 @@ void CDasherControl::ScanColourFiles(std::vector<std::string> &vFileList) {
   colourglob = g_pattern_spec_new("colour*xml");
 
   directory = g_dir_open(GetStringParameter(SP_SYSTEM_LOC).c_str(), 0, NULL);
-  while((filename = g_dir_read_name(directory))) {
-    if(colour_filter(filename, colourglob)) {
-      vFileList.push_back(filename);
+
+  if(directory) {
+    while((filename = g_dir_read_name(directory))) {
+      if(colour_filter(filename, colourglob)) {
+	vFileList.push_back(filename);
+      }
     }
+    g_dir_close(directory);
   }
-  g_dir_close(directory);
 
   directory = g_dir_open(GetStringParameter(SP_USER_LOC).c_str(), 0, NULL);
-  while((filename = g_dir_read_name(directory))) {
-    if(colour_filter(filename, colourglob)) {
-      vFileList.push_back(filename);
+
+  if(directory) {
+    while((filename = g_dir_read_name(directory))) {
+      if(colour_filter(filename, colourglob)) {
+	vFileList.push_back(filename);
+      }
     }
+    g_dir_close(directory);
   }
-  g_dir_close(directory);
 
   // FIXME - need to delete glob?
 }
 
 CDasherControl::~CDasherControl() {
-
   WriteTrainFileFull();
 
   // Delete the input devices
@@ -222,11 +228,6 @@ CDasherControl::~CDasherControl() {
     m_pMouseInput = NULL;
   }
 
-//   if(m_pSocketInput != NULL) {
-//     m_pSocketInput->Unref();
-//     m_pSocketInput = NULL;
-//   }
-
   if(m_pPangoCache != NULL) {
     delete m_pPangoCache;
     m_pPangoCache = NULL;
@@ -271,17 +272,22 @@ GArray *CDasherControl::GetAllowedValues(int iParameter) {
     // For internal glib reasons we need to make a variable and then
     // pass - we can't use the iterator directly
     const char *pTemp(it->c_str());
-    g_array_append_val(pRetVal, pTemp);
+    char *pTempNew = new char[strlen(pTemp) + 1];
+    strcpy(pTempNew, pTemp);
+    g_array_append_val(pRetVal, pTempNew);
   }
 
   return pRetVal;
 }
 
-void CDasherControl::RealizeCanvas() {
+void CDasherControl::RealizeCanvas(GtkWidget *pWidget) {
   // Start the timer loops as everything is set up
   // Aim for 20 frames per second
-
-  g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE, 50, timer_callback, this, NULL);
+  if(!g_bHaveTimer) {
+    g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE, 50, timer_callback, this, NULL);
+    g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE, 5000, long_timer_callback, this, NULL);
+    g_bHaveTimer = true;
+  }
 }
 
 int CDasherControl::CanvasConfigureEvent() {
@@ -302,7 +308,7 @@ void CDasherControl::ExternalEventHandler(Dasher::CEvent *pEvent) {
     HandleParameterNotification(pEvt->m_iParameter);
   }
   // TODO: Horrible - just keep events here
-  else if((pEvent->m_iEventType >= 2) && (pEvent->m_iEventType <= 8)) {
+  else if((pEvent->m_iEventType >= 2) && (pEvent->m_iEventType <= 9)) {
     HandleEvent(pEvent);
   }
 
@@ -322,7 +328,7 @@ void CDasherControl::WriteTrainFile(const std::string &strNewText) {
 void CDasherControl::ExternalKeyDown(int iKeyVal) {
   if(m_pKeyboardHelper) {
     int iButtonID(m_pKeyboardHelper->ConvertKeycode(iKeyVal));
-    
+
     if(iButtonID != -1)
       KeyDown(get_time(), iButtonID);
   }
@@ -341,27 +347,11 @@ void CDasherControl::HandleParameterNotification(int iParameter) {
 
   switch(iParameter) {
   case SP_DASHER_FONT:
-    m_pPangoCache->ChangeFont(GetStringParameter(SP_DASHER_FONT));
-    Redraw(true);
-    break;
-  case LP_MAX_BITRATE:
-    gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_pSpin), GetLongParameter(LP_MAX_BITRATE) / 100.0);
-    break;
-  case BP_SHOW_SLIDER:
-    if(m_pSpeedFrame != NULL) {
-      if(GetBoolParameter(BP_SHOW_SLIDER)) {
-        gtk_widget_show(GTK_WIDGET(m_pSpeedFrame));
-
-        gtk_range_set_value(GTK_RANGE(m_pSpeedHScale), GetLongParameter(LP_MAX_BITRATE) / 100.0);
-      }
-      else {
-        gtk_widget_hide(GTK_WIDGET(m_pSpeedFrame));
-      }
+    if(m_pPangoCache) {
+      m_pPangoCache->ChangeFont(GetStringParameter(SP_DASHER_FONT));
+      ScheduleRedraw();
     }
     break;
-  case SP_ALPHABET_ID:
-    PopulateAlphabetCombol();
-    break;
   case BP_GLOBAL_KEYBOARD:
     if(m_pKeyboardHelper)
       m_pKeyboardHelper->Grab(GetBoolParameter(BP_GLOBAL_KEYBOARD));
@@ -372,50 +362,6 @@ void CDasherControl::HandleParameterNotification(int iParameter) {
   g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_changed", iParameter);
 }
 
-void CDasherControl::PopulateAlphabetCombol() {
-  for( int i(m_iComboCount - 1); i >=0; --i)
-    gtk_combo_box_remove_text(GTK_COMBO_BOX(m_pCombo), i);
-  
-  m_iComboCount = 0;
-  
-  const char *szValue;
-  
-  szValue = GetStringParameter(SP_ALPHABET_ID).c_str();
-  if(strlen(szValue) > 0) {
-    gtk_combo_box_append_text(GTK_COMBO_BOX(m_pCombo), szValue);
-    ++m_iComboCount;
-  }
-  
-  szValue = GetStringParameter(SP_ALPHABET_1).c_str();
-  if(strlen(szValue) > 0) {
-    gtk_combo_box_append_text(GTK_COMBO_BOX(m_pCombo), szValue);
-    ++m_iComboCount;
-  }
-  
-  szValue = GetStringParameter(SP_ALPHABET_2).c_str();
-  if(strlen(szValue) > 0) {
-    gtk_combo_box_append_text(GTK_COMBO_BOX(m_pCombo), szValue);
-    ++m_iComboCount;
-  }
-  
-  szValue = GetStringParameter(SP_ALPHABET_3).c_str();
-  if(strlen(szValue) > 0) {
-    gtk_combo_box_append_text(GTK_COMBO_BOX(m_pCombo), szValue);
-    ++m_iComboCount;
-  }
-  
-  szValue = GetStringParameter(SP_ALPHABET_4).c_str();
-  if(strlen(szValue) > 0) {
-    gtk_combo_box_append_text(GTK_COMBO_BOX(m_pCombo), szValue);
-    ++m_iComboCount;
-  }
-  
-  gtk_combo_box_append_text(GTK_COMBO_BOX(m_pCombo), "More Alphabets...");
-  ++m_iComboCount;
-
-  gtk_combo_box_set_active(GTK_COMBO_BOX(m_pCombo), 0);
-}
-
 
 void CDasherControl::HandleEvent(CEvent *pEvent) {
   if(pEvent->m_iEventType == 2) {
@@ -429,6 +375,12 @@ void CDasherControl::HandleEvent(CEvent *pEvent) {
       // Delete event
       g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_edit_delete", pEditEvent->m_sText.c_str());
     }
+    else if(pEditEvent->m_iEditType == 10) {
+      g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_edit_convert");
+    }
+    else if(pEditEvent->m_iEditType == 11) {
+      g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_edit_protect");
+    }
   }
   else if(pEvent->m_iEventType == 3) {
     CEditContextEvent *pEditContextEvent(static_cast < CEditContextEvent * >(pEvent));
@@ -462,18 +414,25 @@ void CDasherControl::HandleEvent(CEvent *pEvent) {
 
     g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_message", &sInfo);
   }
+  else if(pEvent->m_iEventType == 9) {
+    CCommandEvent *pCommandEvent(static_cast<CCommandEvent *>(pEvent));
+    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_command", pCommandEvent->m_strCommand.c_str());
+  }
 };
 
 int CDasherControl::TimerEvent() {
+
+  //  return 1;
+
   int x, y;
 
   gdk_window_get_pointer(m_pCanvas->window, &x, &y, NULL);
   m_pMouseInput->SetCoordinates(x, y);
 
-  NewFrame(get_time());
+  NewFrame(get_time(), false);
 
   // Update our UserLog object about the current mouse position
-  CUserLog* pUserLog = GetUserLogPtr();
+  CUserLogBase* pUserLog = GetUserLogPtr();
   if (pUserLog != NULL) {  
       // We want current canvas and window coordinates so normalization
       // is done properly with respect to the canvas.
@@ -507,6 +466,17 @@ int CDasherControl::TimerEvent() {
   // See CVS for code which used to be here
 }
 
+int CDasherControl::LongTimerEvent() {
+//   std::cout << "Framerate: " << GetFramerate() << std::endl;
+//   std::cout << "Render count: " << GetRenderCount() << std::endl;
+  return 1;
+}
+
+gboolean CDasherControl::ExposeEvent() {
+  NewFrame(get_time(), true);
+  return 0;
+}
+
 gboolean CDasherControl::ButtonPressEvent(GdkEventButton *event) {
 
   // Take the focus if we click on the canvas
@@ -521,11 +491,11 @@ gboolean CDasherControl::ButtonPressEvent(GdkEventButton *event) {
 
   gtk_widget_grab_focus(GTK_WIDGET(m_pCanvas));
   g_signal_emit_by_name(GTK_OBJECT(m_pCanvas), "focus_in_event", GTK_WIDGET(m_pCanvas), focusEvent, NULL, &returnType);
-
+  
   if(event->type == GDK_BUTTON_PRESS)
-    KeyDown(get_time(), 100);
+    HandleClickDown(get_time(), (int)event->x, (int)event->y);
   else if(event->type == GDK_BUTTON_RELEASE)
-    KeyUp(get_time(), 100);
+    HandleClickUp(get_time(), (int)event->x, (int)event->y);
 
   return false;
 }
@@ -546,8 +516,6 @@ gint CDasherControl::KeyReleaseEvent(GdkEventKey *event) {
       SetLongParameter(LP_BOOSTFACTOR, 100);
   }
   else {
-    int iKeyVal;
-
     if(m_pKeyboardHelper) {
       int iKeyVal(m_pKeyboardHelper->ConvertKeycode(event->keyval));
       
@@ -565,8 +533,6 @@ gint CDasherControl::KeyPressEvent(GdkEventKey *event) {
   else if((event->keyval == GDK_Control_L) || (event->keyval == GDK_Control_R))
     SetLongParameter(LP_BOOSTFACTOR, 25);
   else {
-    int iKeyVal;
-
     if(m_pKeyboardHelper) {
       int iKeyVal(m_pKeyboardHelper->ConvertKeycode(event->keyval));
       
@@ -574,43 +540,9 @@ gint CDasherControl::KeyPressEvent(GdkEventKey *event) {
 	KeyDown(get_time(), iKeyVal);
     }
   }
-
-//   switch (event->keyval) {
-//   case GDK_space:
-//     KeyDown(get_time(), 0);
-//     break;
-//   case GDK_Shift_L:
-//   case GDK_Shift_R: //deliberate fall through
-//     SetLongParameter(LP_BOOSTFACTOR, 175);
-//     break;
-//   case GDK_Control_L:
-//   case GDK_Control_R: //deliberate fall through
-//     SetLongParameter(LP_BOOSTFACTOR, 25);
-//     break;
-//   case GDK_a:
-//     KeyDown(get_time(), 1);
-//     break;
-//   case GDK_s:
-//     KeyDown(get_time(), 2);
-//     break;
-//   case GDK_w:
-//     KeyDown(get_time(), 3);
-//     break;
-//   case GDK_x:
-//     KeyDown(get_time(), 4);
-//     break;
-//   }
   return 0;
 }
 
-void CDasherControl::SliderEvent() {
-  //  int iNewValue( static_cast<int>(round(gtk_range_get_value(GTK_RANGE(m_pSpeedHScale)) * 100)));
-  int iNewValue( static_cast<int>(round(gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(m_pSpin)) * 100)));
-
-  if(GetLongParameter(LP_MAX_BITRATE) != iNewValue)
-    SetLongParameter(LP_MAX_BITRATE, iNewValue);
-}
-
 void CDasherControl::CanvasDestroyEvent() {
   // Delete the screen
 
@@ -620,22 +552,10 @@ void CDasherControl::CanvasDestroyEvent() {
   }
 }
 
-void CDasherControl::AlphabetComboChanged() {
-  // std::cout << "Alphabet changed to: " << gtk_combo_box_get_active_text(GTK_COMBO_BOX(m_pCombo)) << std::endl;
-
-  if(!strcmp("More Alphabets...",  gtk_combo_box_get_active_text(GTK_COMBO_BOX(m_pCombo)))) {
-    gtk_combo_box_set_active(GTK_COMBO_BOX(m_pCombo), 0);
-    
-    g_signal_emit_by_name(GTK_OBJECT(m_pDasherControl), "dasher_request_settings");
-  }
-  else if(strcmp(GetStringParameter(SP_ALPHABET_ID).c_str(),  gtk_combo_box_get_active_text(GTK_COMBO_BOX(m_pCombo))))
-    SetStringParameter(SP_ALPHABET_ID, gtk_combo_box_get_active_text(GTK_COMBO_BOX(m_pCombo)));
-}
-
 // Tell the logging object that a new user trial is starting.
 void CDasherControl::UserLogNewTrial()
 {
-  CUserLog* pUserLog = GetUserLogPtr();
+  CUserLogBase* pUserLog = GetUserLogPtr();
   if (pUserLog != NULL) { 
     pUserLog->NewTrial();
   }
@@ -667,12 +587,9 @@ int CDasherControl::colour_filter(const gchar *filename, GPatternSpec *colourglo
 // here.
 
 extern "C" void realize_canvas(GtkWidget *widget, gpointer user_data) {
-  static_cast < CDasherControl * >(user_data)->RealizeCanvas();
+  static_cast < CDasherControl * >(user_data)->RealizeCanvas(widget);
 }
 
-extern "C" void speed_changed(GtkHScale *hscale, gpointer user_data) {
-  static_cast < CDasherControl * >(user_data)->SliderEvent();
-}
 
 extern "C" gboolean button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer data) {
   return static_cast < CDasherControl * >(data)->ButtonPressEvent(event);
@@ -698,6 +615,6 @@ extern "C" gboolean canvas_focus_event(GtkWidget *widget, GdkEventFocus *event,
   return static_cast < CDasherControl * >(data)->FocusEvent(widget, event);
 }
 
-extern "C" void alphabet_combo_changed(GtkWidget *pWidget, gpointer pUserData) {
-  static_cast<CDasherControl*>(pUserData)->AlphabetComboChanged();
+extern "C" gint canvas_expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer data) {
+  return ((CDasherControl*)data)->ExposeEvent();
 }
