@@ -9,9 +9,10 @@
 #include "WinCommon.h"
 #include <guiddef.h>
 #include <Oleacc.h>
-#include <Textstor.h>
-#include <Msctf.h>
-#include <Msaatext.h>
+// Don't seem to be needed and not in VC8 platform SDK, kdv
+//#include <Textstor.h>
+//#include <Msctf.h>
+//#include <Msaatext.h>
 #include <Htmlhelp.h>
 
 #include "DasherWindow.h"
@@ -212,7 +213,7 @@ void CDasherWindow::Main()
 {
 	DASHER_ASSERT_VALIDPTR_RW(m_pDasher);
 	m_pDasher->Main();
-	Sleep(20); // limits framerate to 50fps
+	Sleep(50); // limits framerate to 50fps
 }
 
 int CDasherWindow::MessageLoop() 
@@ -254,6 +255,7 @@ void CDasherWindow::Show(int nCmdShow)
   InvalidateRect( NULL, FALSE);
   if(!LoadWindowState())
     ShowWindow(nCmdShow);       // Now set up. Kill splash screen and display main window
+
 }
 
 void CDasherWindow::SaveWindowState() const 
@@ -499,7 +501,7 @@ LRESULT CDasherWindow::OnCommand(UINT message, WPARAM wParam, LPARAM lParam, BOO
 			  m_pEdit->New();
 		  // Selecting file->new indicates a new trial to our user logging object
 		  if (m_pDasher != NULL) {
-			  CUserLog* pUserLog = m_pDasher->GetUserLogPtr();
+			  CUserLogBase* pUserLog = m_pDasher->GetUserLogPtr();
 			  if (pUserLog != NULL)
 				  pUserLog->NewTrial();
 		  }
